/*
 * Digimetrie_Sci.c
 * 
 * Copyright (c) 2011, Digimetrie. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

/* ==================================================================== */
#include "Digitool.h"
#include "Digimetrie_Sci.h"
#include "limits.h"
/* ==================================================================== */

#define FIXED_CARTE_MODE 0




//---------------------------------------------------------------------------------
static int b_reset_called = 0;
static int b_init_done = 0;
static int board_id = 0;	// SI 0 => init non faite ou  refaire
static HMODULE board_handle = NULL;
static WORD nb_boards;

/*
#ifdef _WIN32
	#ifdef _WIN64
		static const char dllDir_subpath[] = "..\\..\\thirdParty\\Win64\\";
	#else
		static const char dllDir_subpath[] = "..\\..\\thirdParty\\Win32\\";
	#endif
#else
	#error Target OS not supported
#endif
*/

static int boards_possibles_ids[] = { USB_NANOLABH, USB2_LAB, USB_LAB12, USB_DIGI_EDUC };
// #define MAX_PATH_LEN 500
// static char dll_work_directory[MAX_PATH_LEN];	// Initialis au lancement de la Dll, AVEC le contient le \ ou / de fin
// static int dll_work_directory_len;	// Longueur du chemin de dll_work_directory

#ifdef MY_DEBUG
#include <stdio.h>
FILE * pFile;

#define D_PRINTF( ... ) 							\
	{												\
		pFile = fopen ("d:\\TEMP\\out.txt","a");	\
		fprintf( pFile, __VA_ARGS__ );				\
		fclose( pFile );							\
	}
#else
#define D_PRINTF( ... )
#endif

//---------------------------------------------------------------------------------
static
short
P_Try_Init_with_Board_Id( int t_board_Id )
{
	int b_res_init_ptrs; //, dll_name_len;
	const char* dll_name;
	
	board_id = 0;
	
	dll_name = Digi_GetCarte_Dll( t_board_Id );
	if( dll_name == NULL )
		return ERR_DT_PARAMETRE;

/* REMOVED : *.dll should be in system path		
	// Construction du chemin de l'ventuelle librairie
	D_PRINTF( "dll_name '%s'\n", dll_name );
	dll_name_len = strlen( dll_name);
	strncpy( &dll_work_directory[dll_work_directory_len], dll_name, dll_name_len );
	dll_work_directory[dll_work_directory_len + dll_name_len] = 0;
	D_PRINTF( "Board Dll '%s'\n", dll_work_directory );
*/

	board_handle = LoadLibrary( dll_name );
	if( board_handle == NULL )
	{
		D_PRINTF( "Board Load Fail\n" );
		nb_boards = 0;
		return ERR_DT_OK;	// Echec du chargement de la DLL => pas une erreur
	}
	
	// Initialisation des pointeurs de fonctions 
	b_res_init_ptrs = Digi_Init_Functions_Pointers( board_handle, dll_name );
	// Si echec de l'init des pointeurs
	if( b_res_init_ptrs == 0 )
	{
		D_PRINTF( "Digi_Init_Functions_Pointers() Fail\n" );
		FreeLibrary( board_handle );
		board_handle = NULL;
		return ERR_DT_INITDLL; // Erreur Non permise de chargement de la Dll
	}

	// Si le pointeur vers pf_Reset_carte est NULL
	if( pf_Reset_carte == NULL )
	{
		D_PRINTF( "ERROR : no pf_Reset_carte() \n" );
		FreeLibrary( board_handle );
		board_handle = NULL;
		return ERR_DT_INITDLL; // Erreur Non permise de chargement de la Dll
	}
	
	// Vois le nombre de carte de ce type dans le systme
	if( pf_Get_Nb_Cartes == NULL )
	{
		D_PRINTF( "ERROR : no pf_Get_Nb_Cartes() \n" );
		return ERR_DT_FONCTION_NDEV;
	}
	nb_boards = pf_Get_Nb_Cartes( t_board_Id );
	
	// Si aucune carte de trouv => un peu de nettoyage
	if( nb_boards == 0 )
	{
		D_PRINTF( "Init Fail : no Zero Board found \n" );
		FreeLibrary( board_handle );
		board_handle = NULL;
		return ERR_DT_OK;
	}
	
	board_id = t_board_Id;
	
	D_PRINTF( "Init Succeed : %i boards\n", nb_boards );
	return ERR_DT_OK;
}

//---------------------------------------------------------------------------------
//! Renvoi ERR_DT_OK ou bien un code d'erreur
short
P_Init()
{
	int i;
	short err;

	b_init_done = 1;	
	
	for( i = 0; i < sizeof(boards_possibles_ids) /sizeof(int) ; i++ )
	{
		// Essaie La Dll associ au cartes de type boards_possibles_ids[i]
		err = P_Try_Init_with_Board_Id( boards_possibles_ids[i] );
		if( err != ERR_DT_OK )
			return err;
		
		// S'il y a des cartes de ce type dans le systme => YES
		if( nb_boards > 0 )
			return ERR_DT_OK;
	}
	
	// Aucune carte d'aucun type permis n'a t trouve dans le systme
	if( board_handle != NULL )
	{
		FreeLibrary( board_handle );
		board_handle = NULL;
	}
	
	D_PRINTF( "P_Init() FAIL\n" );
	return ERR_DT_INIT;
}

//---------------------------------------------------------------------------------
#include <windows.h>
int 
WINAPI 
DllMain (	  HINSTANCE hInstance
		    , DWORD reason
		    , PVOID pvReserved
		)
{
	int i;
	
	switch (reason) 
    {
		case DLL_PROCESS_ATTACH:
		{
			D_PRINTF( "DLL_PROCESS_ATTACH '%i'\n", hInstance );
			
/*			
			dll_work_directory_len = GetModuleFileName( hInstance, dll_work_directory, MAX_PATH_LEN );

			D_PRINTF( "dll_work_directory '%s'\n", dll_work_directory );
			
			// Si, impossible d'obtenir le chemin de digimetrie.dll
			if( dll_work_directory_len == 0 )
			{
				D_PRINTF( "DllMain failed 'impossible d'obtenir le chemin de digimetrie.dll'\n" );
				return 0;
			}
			
			// Si, le buffer qui a recu le chemin de digimetrie.dll tait trop petit
			if( dll_work_directory_len == MAX_PATH_LEN )
			{
				D_PRINTF( "DllMain failed 'buffer chemin trop petit'\n" );
				return 0;
			}
			
			// Extraction du rpertoire
			for( i = dll_work_directory_len -1; i > 0; i-- )
			{
				if( dll_work_directory[i] == '\\' )
				{
					dll_work_directory[i+1] = 0;
					break;
				}
				else
					dll_work_directory_len--;
			}
			
			// Si, le / ou \ du rpertoire contenant la Dll n'a pas t trovu
			if( i == 0 )
			{
				D_PRINTF( "n'a pas trouve le separateur dans le chemin\n" );
				return 0;
			}

			strncpy( &dll_work_directory[dll_work_directory_len], dllDir_subpath, sizeof(dllDir_subpath) - 1 );
			dll_work_directory_len += sizeof(dllDir_subpath) - 1;
			dll_work_directory[dll_work_directory_len] = 0;
			D_PRINTF( "dll_work_directory '%s'\n", dll_work_directory );
*/		

			// Sinon, tout est ok, on lance l'init initiale
			P_Init();
			D_PRINTF( "board_handle '%i'\n", board_handle );

			break;
		}
		case DLL_PROCESS_DETACH:
		  break;
		case DLL_THREAD_ATTACH:
		  break;
		case DLL_THREAD_DETACH:
		  break;
		}
		return 1;
}

//---------------------------------------------------------------------------------
static 
short
P_call_precheck( void* pf_fonct )
{
	short err;
	int i;
	
	if( b_init_done == 0 )
		err = P_Init();
		
	if( nb_boards == 0 )
		return ERR_DT_INITDLL;
	
	if( pf_fonct == NULL )
		return ERR_DT_FONCTION_NDEV;
	
	if( b_reset_called == 0 )
	{
		for( i =0; i < nb_boards; i++ )
		{
			pf_Reset_carte( 0, 0, board_id, i );
		}
		b_reset_called = 1;
	}
	
	return ERR_DT_OK;
}	

//---------------------------------------------------------------------------------
// Return :
//    > 0 : le nombre de stations
// 	  <= 0 : l'oppos du code d'erreur ( redresser)
int
digi_Reinit()
{
	short err;
	// On cloture l'ventuelle init prcdente
	if( board_handle != NULL )
	{
		D_PRINTF( "digi_Reinit() : Close previous 'board_handle'\n" );
		FreeLibrary( board_handle );
		board_handle = NULL;
		nb_boards = 0;
	}
	
	b_reset_called = 0;
	b_init_done = 0;
	err = P_Init();
	if( err != ERR_DT_OK )
	{
		D_PRINTF( "digi_Reinit() : Error Code given by P_Init() %i\n", err );
		return -err;
	}
		
	D_PRINTF( "digi_Reinit() : %i boards found \n", nb_boards );
	return nb_boards;
}

//---------------------------------------------------------------------------------
int
digi_AInf(int num_carte, int num_voie, int gain, float *res )
{
	short err;
	if( (err = P_call_precheck( (void*) pf_Ainf ) ) != ERR_DT_OK )
		return err;
	
	D_PRINTF( "digi_AInf() : num voie %i, gain %i, board_id %i, num_carte %i\n"
					, num_voie
					, gain
					, board_id
					, num_carte
					);
	
	return pf_Ainf(
		res,  						//!< rsultat en tension de la conversion 
		(USHORT) num_voie,        	//!< voie analogique slectionne
		(USHORT) gain,        		//!< valeur du gain (voir selon la carte)
		FIXED_CARTE_MODE,        	//!< selection niveau et configuration d'entree (voir selon la carte)
		0,         					//!< extension toujour a 0
		board_id,       			//!< Identifiant du type de carte
		(USHORT) num_carte      	//!< FIXDOC adresse de la carte
		);
}

//---------------------------------------------------------------------------------
int
digi_AInf_Scan(int num_carte, int nb_voies, USHORT arr_gain[], float *arr_res, int _frequence_hz, int nb_acquisitions )
{
	ULONG frequence_hz;
	short err;
	if( (err = P_call_precheck( (void*) pf_Ainfscan ) ) != ERR_DT_OK )
		return err;
	
	frequence_hz = (ULONG) _frequence_hz;
	return pf_Ainfscan(
			0,						//!<  Stockage a partir de la premiere voie du tampon utilisateur	      
			0,						//!<  Acquisition a partir de la voie V sur la carte	      
			nb_voies,				//!<  Nombre de voies maximum sur le tampon utilisateur     
			nb_voies,				//!<  Nombre de voies a acquerir sur la carte     
			arr_gain,				//!<  Configuration du gain pour chaque voie Gain[nb_voies](voir selon la carte)	 
			nb_acquisitions,		//!<  nombre d'echantillons  acquerir(tableau de donnees)     
			&frequence_hz,  		//!<   la frquence (en Hertz)   
			DT_AD_TIM,				//!<  mode de configuration generateur interne + dclenchement 
			arr_res,				//!<  tableau de donnees tension, zone de stockage 
			0,			    		//!<  extension, a 0 si pas de thread d'acquisition
			board_id, 			//!<  nom de la carte		      
			(USHORT) num_carte		//!<  adresse de la carte		      
		);
}

//---------------------------------------------------------------------------------
int
digi_AOutf(int num_carte, int num_voie, float val )
{
	short err;
	if( (err = P_call_precheck( (void*) pf_Aoutf ) ) != ERR_DT_OK )
		return err;
	
	return pf_Aoutf(
				 val,       			//!< Valeur en tension  crire 
				 (USHORT) num_voie,     //!< voie de sortie analogique slectionne
				 FIXED_CARTE_MODE,      //!< selection niveau et configuration de sortie(voir selon la carte)
				 0,       				//!< extension toujour a 0
				 board_id,     	//!< Identifiant du type de carte
				 (USHORT) num_carte	   	//!< FIXDOC adresse de la carte
				 );
}

//---------------------------------------------------------------------------------
int
digi_AOutf_Scan(int num_carte, int nb_voies, float* arr_val, int nb_restitutions, int _frequence )
{
	ULONG frequence;
	short err;
	if( (err = P_call_precheck( (void*) pf_Aoutfscan ) ) != ERR_DT_OK )
		return err;
	
	frequence = (ULONG) _frequence;
	
	return pf_Aoutfscan(
			0,						//!<  lecture a partir de la premiere voie du tampon utilisateur	
			0,         				//!<  restitution a partir de la voie V sur la carte
			nb_voies,				//!<  Nombre de voies maximum sur le tampon utilisateur
			nb_voies,      			//!<  restitution sur N voies sur la carte
			nb_restitutions,	  	//!<  nombre d'echantillons  restituer(tableau de donnees)
			&frequence,	  			//!<   la frquence (en Hertz)
			0,       				//!<  toujours a 0
			DT_AD_TIM,				//!<  mode de configuration generateur interne + dclenchement  (cf. les macros DT_AD_...)
			NULL ,					//!<  Inutilise
			arr_val,				//!<  tableau de donnees tension, zone de stockage
			0,              		//!<  extension, toujour a 0 
			board_id, 	 		//!<  nom de la carte
			(USHORT) num_carte		//!<  adresse de la carte
		);
}

//---------------------------------------------------------------------------------
int 
digi_DIn(int num_carte, int num_voie, int *res )
{
	USHORT tres;
	short err;
	
	if( (err = P_call_precheck( (void*) pf_Din ) ) != ERR_DT_OK )
		return err;
	
	if( pf_Dinit == NULL )
		return ERR_DT_FONCTION_NDEV;
	if( (err = pf_Dinit(num_voie, DT_DINIT_IN, 0, num_carte, board_id) ) != 0 )
		return err;
	
	 err = pf_Din(	&tres,
				(USHORT) num_voie,
				0, 						//!< numero du composant : un seul sur les cartes utilises pour Scilab
				board_id,        	
				(USHORT) num_carte 
				);
	*res = tres;
	return err;
}

//---------------------------------------------------------------------------------
int 
digi_BIn(int num_carte, int num_voie, int num_ligne, int *res )
{
	USHORT tres;
	short err;
	
	if( (err = P_call_precheck( (void*) pf_BIn ) ) != ERR_DT_OK )
		return err;
	
	if( pf_Dinit == NULL )
		return ERR_DT_FONCTION_NDEV;
	if( (err = pf_Dinit(num_voie, DT_DINIT_IN, 0, num_carte, board_id) ) != 0 )
		return err;
	
	err = pf_BIn(	&tres,   				
				(USHORT) num_ligne,      
				(USHORT) num_voie,      
				0, 					//!< numero du composant : un seul sur les cartes utilises pour Scilab
				board_id,
				(USHORT) num_carte
				);
	*res = tres;
	return err;
}

//---------------------------------------------------------------------------------
int
digi_DOut(int num_carte, int num_voie, int _val )
{
	USHORT val;
	short err;
	if( (err = P_call_precheck( (void*) pf_Dout ) ) != ERR_DT_OK )
		return err;
	
	if( _val > USHRT_MAX )
		return ERR_DT_PARAMETRE;

	if( pf_Dinit == NULL )
		return ERR_DT_FONCTION_NDEV;
	if( (err = pf_Dinit(num_voie, DT_DINIT_OUT, 0, num_carte, board_id) ) != 0 )
		return err;
	
	val = (USHORT) _val;
	
	// On inverse la donne pour inverser le comportement invers des Dlls des cartes UsbLab pour les voies digitales
	val = ~val;
	
	return pf_Dout(
				(USHORT) val,
				(USHORT) num_voie,
				0, 						//!< numero du composant : un seul sur les cartes utilises pour Scilab
				board_id,
				(USHORT) num_carte
				);
}

//---------------------------------------------------------------------------------
int
digi_BOut(int num_carte, int num_voie, int num_ligne, int _val_ligne )
{
	USHORT val;
	short err;
	if( (err = P_call_precheck( (void*) pf_BOut ) ) != ERR_DT_OK )
		return err;
	
	if( pf_Dinit == NULL )
		return ERR_DT_FONCTION_NDEV;
	if( (err = pf_Dinit(num_voie, DT_DINIT_OUT, 0, num_carte, board_id) ) != 0 )
		return err;

	// On inverse la donne pour inverser le comportement invers des Dlls des cartes UsbLab pour les voies digitales
	val = ( (_val_ligne == 0) ? 1 : 0);
		
	return pf_BOut(
				(USHORT) val,
				(USHORT) num_ligne,
				(USHORT) num_voie,
				0, 						//!< numero du composant : un seul sur les cartes utilises pour Scilab
				board_id,
				(USHORT) num_carte
				);
}



//---------------------------------------------------------------------------------
const char*
digi_get_Error_String( int err_num )
{
	return Digi_GetErrorString( err_num );
}


//---------------------------------------------------------------------------------
int
digi_TestParam( int num_carte, int num_voie, int typeTest )
{
	short err;
	USHORT Wflag,Wval;
	float bande_passante;
	ULONG ext;
	CF_CARTE cf_carte;

	if( (err = P_call_precheck( (void*) pf_GetInfo_Reseau ) ) != ERR_DT_OK )
		return err;
		
	// Test le numero de carte
	if( typeTest == 0)
	{
		err = pf_GetInfo_Reseau(&Wflag,&Wval,&bande_passante,0,board_id, num_carte);
		return err;
	}
	
	if( pf_Get_Conf == NULL )
		return ERR_DT_FONCTION_NDEV;
	if( (err = pf_Get_Conf( &cf_carte )) != ERR_DT_OK )
		return err;
	
	switch( typeTest )
	{
		// Teste la voie spcifie en AIN
		case 1:
			// NOTE :
			// Les cartes USB on un nombre de port Fix pour le AIN et le AOUT
			if( cf_carte.adnmaxvoies <= num_voie )
				return ERR_DT_VOIE;
			return ERR_DT_OK;
			
		// Teste la voie spcifie en AOUT
		case 2:
			// NOTE :
			// Les cartes USB on un nombre de port Fix pour le AIN et le AOUT
			if( cf_carte.danmaxvoies <= num_voie )
				return ERR_DT_VOIE;
			return ERR_DT_OK;
			
		// Teste la voie spcifie en DIN
		case 3:
		// Teste la voie spcifie en DOUT
		case 4:
			// NOTE :
			// Les cartes USB on leur port Digitaux bidirectionnels, donc on ne peut pas viter  l'user ses propres tourderies
			if( cf_carte.tormaxports <= num_voie )
				return ERR_DT_VOIE;
			return ERR_DT_OK;

			
		// Type de test inconnu
		default:
			return ERR_DT_PARAMETRE;
	}
	
	
	return ERR_DT_BUG;
}	

/* ==================================================================== */
