/*
 * Digimetrie_Scicos.c
 * 
 * Copyright (c) 2011, Digimetrie. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */

 
/* ==================================================================== */
#include <scicos_block4.h>
#include "Digimetrie_Sci.h"
/* ==================================================================== */

#define F_IN(n, i) ((GetRealInPortPtrs(block, n+1))[(i)])
#define F_IN_0 (F_IN(0,0)) // first float input value

#define F_OUT(n, i) ((GetRealOutPortPtrs(block, n+1))[(i)])
#define F_OUT_0 (F_OUT(0,0))


  /* 
  int block->nevprt;
  int block->nz;
  double* block->z;
  int block->nx;
  double* block->x;
  double* block->xd;
  double* block->res;
  int block->nin;
  int *block->insz;
  double **block->inptr;
  int block->nout;
  int *block->outsz;
  double **block->outptr;
  int block->nevout;
  int block->nrpar;
  double *block->rpar;
  int block->nipar;
  int *block->ipar;
  int block->ng;
  double *block->g;
  int *block->jroot;
  char block->label[41];
  */

 


//--------------------------------------------
void 
digi_scicos_AInf( scicos_block *block,int flag )
{
    float data=1;
	short err;
	
	#ifdef FIXED_CARTE_VALUES
	int num_carte = 0;
	int num_voie = 0;
	int gain = 0;
	#else
	int num_carte = block->ipar[0];
	int num_voie = block->ipar[1];
	int gain = block->ipar[2];
	#endif
	
	switch( flag )
	{
		// compute the derivative of the continuous time state
		case 0:
			// Not used...
			break;
		
		// compute the outputs of the block
		case 1:
		{
			err = digi_AInf(num_carte, num_voie, gain, &data );
			if( err != 0 )
				Coserror( digi_get_Error_String(err) );
			else
				F_OUT_0 = data;
			break;
		}
		
		// compute zero crossing surfaces and set modes
		case 9:
			// Not used...
			break;
	}
}



//--------------------------------------------
void 
digi_scicos_BIn( scicos_block *block,int flag )
{
    int data=1;
	
	#ifdef FIXED_CARTE_VALUES
	int num_carte = 0;
	int num_voie = 0;
	int num_ligne = 0;
	#else
	int num_carte = block->ipar[0];
	int num_voie = 0;
	int num_ligne = block->ipar[1];
	#endif
	
	switch( flag )
	{
		// compute the derivative of the continuous time state
		case 0:
			// Not used...
			break;
		
		// compute the outputs of the block
		case 1:
		{
			short err = digi_BIn(num_carte, num_voie, num_ligne, &data );
			if( err != 0 )
				Coserror( digi_get_Error_String(err) );
			else
				F_OUT_0 = data;
			break;
		}
		
		// compute zero crossing surfaces and set modes
		case 9:
			// Not used...
			break;
	}
}



//--------------------------------------------
void 
digi_scicos_AOutf( scicos_block *block,int flag )
{
	// block->outptr[0]=&data;
	// block->outptr[0][0]=2;
	short err;
	#ifdef FIXED_CARTE_VALUES
	int num_carte = 0;
	int num_voie = 0;
	#else
	int num_carte = block->ipar[0];
	int num_voie = block->ipar[1];
	#endif
	
	/* get the ptrs of the first real regular input port */
	double val = F_IN_0;
	
	switch( flag )
	{
		// compute the derivative of the continuous time state
		case 0:
			// Not used...
			break;
		
		// compute the outputs of the block
		case 1:
		{
			err = digi_AOutf(num_carte, num_voie, (float) val );
			if( err != 0 )
				Coserror( digi_get_Error_String(err) );
			break;
		}
		
		// compute zero crossing surfaces and set modes
		case 9:
			// Not used...
			break;
	}
}


//--------------------------------------------
void 
digi_scicos_BOut( scicos_block *block,int flag )
{
	short err;
	#ifdef FIXED_CARTE_VALUES
	int num_carte = 0;
	int num_voie = 0;
	int num_ligne = 0;
	#else
	int num_carte = block->ipar[0];
	int num_voie = 0;
	int num_ligne = block->ipar[1];
	#endif
	int data = (int) F_IN_0;
	
	switch( flag )
	{
		// compute the derivative of the continuous time state
		case 0:
			// Not used...
			break;
		
		// compute the outputs of the block
		case 1:
		{
			err = digi_BOut(num_carte, num_voie, num_ligne, data );
			if( err != 0 )
				Coserror( digi_get_Error_String(err) );
			break;
		}
		
		// compute zero crossing surfaces and set modes
		case 9:
			// Not used...
			break;
	}
}

